/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data;

/**
 * 
 * Informacje o pojedynczym graczu - uczestniku meczu
 *
 */

public final class Player implements Comparable<Player>, Cloneable {
    
    /** identyfikator gracza (dodatni) */
    public int id;
    
    public int score; 
    public String nickname;
    public PlayerColor color;
    public State state;
    
    /** czy gracz bierze udział w bieżącej rundzie */
    public boolean active;
    
    /** Kiedy gracz ginie (lub null) */
    public RoundTime death;

    /** konstruktor do użycia przez deserializację */
    public Player() {} 
    
    public Player(int id, String nickname, PlayerColor color) 
    {
        assert id > 0;
        this.score = 0;
        this.nickname = nickname;
        this.color = color;
        this.id = id;
        this.state = State.PLAYING;
    }

    public enum State {
        PLAYING, SPECTATING, DISCONNECTED;
    }
    
    /** Sprawdza, czy na początku podanego kroku gracz jeszcze żyje. */
    public boolean isAlive(int stepNum) {
        assert state == State.PLAYING || state == State.DISCONNECTED;
        assert active;
        return death == null || death.t >= stepNum;
    }

    public boolean isPlaying() {
        return state == State.PLAYING;
    }
    
    public int compareTo(Player p) {
        return id - p.id;
    }

    public int hashCode() {
        return id;
    }

    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Player other = (Player) obj;
        if (id != other.id)
            return false;
        return true;
    }
    
    // Nie używamy tu Cloneable, bo podobno nie należy...
    /** Zwraca kopię Playera. */
    public Player getCopy() {
        try {
            return (Player)clone();
        } catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
    
    @Override
    public String toString() {
    	return "Player:" + nickname + "[" + state + ",color=" + color + 
    	                                            ",score=" + score + "]";
    }
}

