/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

import netacka3d.data.Player.State;
import netacka3d.misc.Logger;

import org.codehaus.jackson.annotate.JsonProperty;

/**
 * Pojemnik na dane graczy. 
 */
public final class PlayersTable {
    
    @JsonProperty
    private TreeMap<Integer, Player> players = new TreeMap<Integer, Player>();

    public PlayersTable() {
    }

    public void addPlayer(Player p) {
        // jesli w tabeli byl juz gracz o podanym id to znaczy ze cos
        // sie popsulo
        assert (!players.containsKey(p.id));
        players.put(p.id, p);    
    }

    public void removePlayer(int id) {
        assert players.containsKey(id);
        players.remove(id);
    }

    public Player getPlayer(int id) {
        return players.get(id);
    }
    public Player getPlayer(String nick) {
        assert nick != null;
        for (Player p: players.values()) // O(n)!
            if (p.nickname.equals(nick))
                return p;
        return null;
    }
    public Player getPlayer(PlayerColor c) {
        assert c != null;
        for (Player p: players.values()) // O(n)!
            if (p.color.equals(c))
                return p;
        return null;
    }
    
    public int getAlivePlayerCount(int stepNum) {
        int n = 0;
        for (Player p: players.values())
            if (p.active && p.isAlive(stepNum))
                ++n;
        return n;
    }
    
    public void resetDeaths() {
        for (Player p : players.values())
            p.death = null;
    }
    
    public void removeDisconnected() {
        Iterator<Integer> it = players.keySet().iterator();
        while (it.hasNext()) {
            int id = it.next();
            if (players.get(id).state == State.DISCONNECTED)
                it.remove();
        }
    }
    
    /** Funkcja pomocnicza dla get*Players */
    private List<Player> _getPlayers(int death) {
       List<Player> ret = new ArrayList<Player>();
       for (Player p: players.values()) {
           if (!p.active)
               continue;
           if (death != -1 && !p.isAlive(death))
               continue;
           ret.add(p);
       }
       return ret;
    }
    
    public List<Player> getPlayers() {
        return new ArrayList<Player>(players.values());
    }
    
    /** Zwróć wszystkich graczy, grających w tej rundzie */
    public List<Player> getActivePlayers() {
        return _getPlayers(-1);
    }
    
    /** Zwróć graczy żywych w danej chwili. */
    public List<Player> getAlivePlayers(int stepNum) {
        assert stepNum >= 0;
        return _getPlayers(stepNum);
    }
    
    public int getPlayerCount() {
        return players.size();
    }
    
    public int getMaxScore() {
        int max = 0;
        for (Player p: players.values())
            if (p.score > max)
                max = p.score;
        return max;
    }
    
    /** O(n^2) :( */
    public void checkConsistency() {
        for (Player p : players.values()) {
            assert p.id > 0;
            assert p.color != null;
            assert p.nickname != null;
            assert p.score >= 0;
            assert p.state != null;
            
            for (Player p2 : players.values()) {
                if (p == p2) 
                    continue;
                assert p.id != p2.id;
                assert p.nickname != p2.nickname;
                assert p.color != p2.color;
            }
        }
    }
    
    /** Zmienia dane gracza, żeby można go bylo dodać.
     * Zwraca null, jeśli się nie da.
     * Modyfikuje podanego playera. */
    public Player resolveConflicts(Player p) {
        if (getPlayer(p.nickname) != null) { // kolizja nicków?
            String newNick;
            int i=1;
            do {
                newNick = p.nickname + "_" + i;
                i++;
            }
            while (getPlayer(newNick) != null);
            Logger.d("zmieniłem playerowi nicka z "+p.nickname+" na "+newNick);
            p.nickname = newNick;
        }
        
        if (getPlayer(p.color) != null) { // kolizja kolorow?
            PlayerColor oldColor = p.color;
            for (PlayerColor c : PlayerColor.values()) {
                if (getPlayer(c) == null) {
                    p.color = c;
                    break;
                }
            }
            if (oldColor == p.color) {
                Logger.w("nie ma wolnego koloru dla gracza "+p.nickname+
                         ", wywalam go");
                return null;
            }
            else
                Logger.d("zmieniłem graczowi "+p.nickname+" kolor z "+
                         oldColor+" na "+p.color);
        }
        return p;
    }
    
    /** Zwraca głęboką kopię (z nowymi Playerami). */
    public PlayersTable getCopy() {
        PlayersTable table = new PlayersTable();
        for (Player p: players.values())
            table.players.put(p.id, p.getCopy());
        return table;
    }
    
    @Override
    public String toString() {
        return "PlayersTable[" + players.toString() + "]";
    }
}

