/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.vecmath.Matrix4f;
import javax.vecmath.Quat4f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

/**
 * Pomocnicze funkcje do obliczeń.
 * 
 * @author Pawel
 *
 */
public class Util3d {
    /** 
     * Zastosuj obrót do wektora, modyfikując wektor w miejscu.
     *  
     *  q' = q * (v.x, v.y, v.z, 0.0) * q*
     *  v = (q'.x, q'.y, q'.z)
     *  
     * @param v wejściowy wektor, modyfikowany w miejscu
     * @param q kwaternion reprezentujący obrót
     */
    public static void applyQuatToVector(Vector3f v, Quat4f q) {
        /*
        Quat4d q1 = new Quat4d(v.x, v.y, v.z, 0.0);
        q1.mul(new Quat4d(-q.x, -q.y, -q.z, q.w));
        q1.mul(q, q1);
        v.set(q1.x, q1.y, q1.z);
        */
        /* Okazało się, że powyższy kod jest zły, bo kwaterniony (Quat4d) się 
         * automatycznie normalizują :/
         */
        // TODO napisać to mnożenie bezpośrednio?
        Matrix4f mat = new Matrix4f(q, new Vector3f(), 1.0f);
        mat.transform(v);
    }
    
    /**
     * Przekonwertuj Matrix4f na tablicę 16 floatów, w formacie dla OpenGL.
     */
    public static float[] matrix4fToFloatArray(Matrix4f mat) {
        /*return new float[] {
            mat.m00, mat.m01, mat.m02, mat.m03,
            mat.m10, mat.m11, mat.m12, mat.m13,
            mat.m20, mat.m21, mat.m22, mat.m23,
            mat.m30, mat.m31, mat.m32, mat.m33
        };*/
        return new float[] {
            mat.m00, mat.m10, mat.m20, mat.m30,
            mat.m01, mat.m11, mat.m21, mat.m31,
            mat.m02, mat.m12, mat.m22, mat.m32,
            mat.m03, mat.m13, mat.m23, mat.m33,
        };
    }
    
    public static int hashVec(Vector3f v) {
        return Float.floatToIntBits(v.x) ^
            Float.floatToIntBits(v.y) ^
            Float.floatToIntBits(v.z);
    }
    
    /** 
     * Zaalokuj ByteBuffer z odpowiednią endianowością (dla grafiki).
     * 
     * @param size wielkość w bajtach
     */
    public static ByteBuffer newNativeByteBuffer(int size) {
        return ByteBuffer.allocateDirect(size).order(ByteOrder.nativeOrder());
    }

    /** 
     * Zaalokuj FloatBuffer z odpowiednią endianowością (dla grafiki).
     * 
     * @param size wielkość we floatach
     */
    public static FloatBuffer newNativeFloatBuffer(int size) {
        return newNativeByteBuffer(Float.SIZE*size).asFloatBuffer();
    }
    
    /**
     * Zaalokuj FloatBuffer i załaduj tablicę floatów.
     */
    public static FloatBuffer newNativeFloatBuffer(float[] array) {
        FloatBuffer buf = 
            newNativeByteBuffer(Float.SIZE*array.length).asFloatBuffer();
        buf.put(array);
        buf.position(0);
        return buf;
    }
    
    /**
     * Zrób ByteBuffer z pikselami (dla grafiki)
     * 
     * @param data dane w formacie ARGB (mogą być modyfikowane)
     * @return bufor w formacie RGBA
     */
    public static ByteBuffer newPixelBuffer(int[] data) {
        for (int i = 0; i < data.length; ++i)
            data[i] = ((data[i]&0x00FFFFFF) << 8) +
                      ((data[i]&0xFF000000) >> 24);
        ByteBuffer buf = ByteBuffer.allocateDirect(data.length*4);
        buf.order(ByteOrder.BIG_ENDIAN);
        buf.asIntBuffer().put(data);
        buf.order(ByteOrder.nativeOrder());
        return buf;
    }

    /** Liczy (p1-p2) x (q1-q2) */
    public static Vector3f crossDelta(Tuple3f p1, Tuple3f p2, 
            Tuple3f q1, Tuple3f q2)
    {
        Vector3f p1p2 = new Vector3f(p1);
        Vector3f q1q2 = new Vector3f(q1);
        Vector3f r = new Vector3f();
        p1p2.sub(p2);
        q1q2.sub(q2);
        r.cross(p1p2, q1q2);
        return r;
    }
}
