/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data.arena;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import netacka3d.data.arena.Mesh.Material;
import netacka3d.misc.FileIO;
import netacka3d.misc.Logger;

/**
 * Importer plików mtl.
 * Opis formatu: http://en.wikipedia.org/wiki/Material_Template_Library
 * 
 * @author mik01aj
 */
public abstract class MtlImporter extends Importer {
    
    public static Map<String, Material> loadFromFile(String filename) 
                                     throws IOException, DataFormatException {

        BufferedReader br = new BufferedReader(
                    new InputStreamReader(FileIO.openFile(filename)));
        
        Map<String, Material> mats = new HashMap<String, Material>();
        Material currentMat = null;
        
        String line;
        while ((line = br.readLine()) != null) {
            
            line = line.trim();
            
            if (line.length() == 0 || line.startsWith("#")) 
                continue;
            
            StringTokenizer st = new StringTokenizer(line, " ");
            String command = st.nextToken();
            
            if (command.startsWith("#")) continue;
            
            if (command.equals("newmtl")) {
                currentMat = new Material(st.nextToken());
                mats.put(currentMat.name, currentMat);
            }
            else if (command.equals("Ka"))
                currentMat.ambientColor = readColor(st);
            else if (command.equals("Kd"))
                currentMat.diffuseColor = readColor(st);
            else if (command.equals("Ks"))
                currentMat.specularColor = readColor(st);
            
            else if (command.equals("map_Ka"))
                currentMat.ambientTexture = st.nextToken();
            else if (command.equals("map_Kd"))
                currentMat.diffuseTexture = st.nextToken();
            else if (command.equals("map_Ks"))
                currentMat.specularTexture = st.nextToken();
            
            else if (command.equals("d") || command.equals("Tr"))
                currentMat.transparency = readFloat(st);
            else if (command.equals("Ns"))
                currentMat.specComponent = readFloat(st);
            else if (command.equals("illum"))
                currentMat.illuminationModel = readInt(st);
            
            else if (command.equals("Ni"))
                st.nextToken(); // optical density (unsupported)
            
            else {
                Logger.w("unknown command: " + command);
                while (st.hasMoreTokens()) 
                    st.nextToken();
            }
            
            assert !st.hasMoreTokens();
        }
        
        return mats;
    }
}
