/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.data.arena;


/**
 * Klasa ładująca pliki w formacie *.obj.
 * 
 * O formacie OBJ: http://www.royriggs.com/obj.html
 * 
 * @author mik01aj
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.vecmath.Point3f;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector3f;

import netacka3d.data.arena.Mesh.FaceBuilder;
import netacka3d.data.arena.Mesh.Material;
import netacka3d.misc.FileIO;
import netacka3d.misc.Logger;

public abstract class ObjImporter extends Importer {

    public static Mesh loadFromFile(String filename) 
                                    throws IOException, DataFormatException {
        
        BufferedReader br = new BufferedReader(
                    new InputStreamReader(FileIO.openFile(filename)));
        
        Mesh mesh = new Mesh();
        Map<String, Material> mats = new HashMap<String, Material> ();
        
        String line;
        while ((line = br.readLine()) != null) {
            
            line = line.trim();
            
            if (line.length() == 0 || line.startsWith("#")) 
                continue;
            
            StringTokenizer st = new StringTokenizer(line, " ");
            String command = st.nextToken();
            
            if (command.equals("v"))
                mesh.addPoint(new Point3f(readFloat(st),
                                          readFloat(st),
                                          readFloat(st)));
            else if (command.equals("vn"))
                mesh.addNormal(new Vector3f(readFloat(st),
                                            readFloat(st),
                                            readFloat(st)));
            else if (command.equals("vt")) 
                mesh.addTexCoord(new Vector2f(readFloat(st),
                                              readFloat(st)));
            
            else if (command.equals("f")) {
                int count = st.countTokens(); // liczba wierzchołków
                
                FaceBuilder fb = mesh.new FaceBuilder();
                
                //for each tuple (vertex, texCoord, normal)...
                for (int i = 0; i < count; i++) {
                    String chars = st.nextToken().replace("//", "/0/");
                    
                    StringTokenizer st2 = new StringTokenizer(chars, "/");
                    int num = st2.countTokens();
                    
                    int vNumber = readInt(st2);
                    int vtNumber = 0;
                    int vnNumber = 0;
                    if (num > 1) vtNumber = readInt(st2);
                    if (num > 2) vnNumber = readInt(st2);
                    
                    fb.addVertex(vNumber, vtNumber, vnNumber);
                }
                
                fb.commit();
            }
            
            else if (command.equals("mtllib"))
                mats.putAll(MtlImporter.loadFromFile(st.nextToken()));
            else if (command.equals("usemtl"))
                mesh.setCurrentMaterial(mats.get(st.nextToken()));
            
            else if (command.equals("s"))
                st.nextToken(); // smooth on/off - not supported
            else if (command.equals("g"))
                st.nextToken(); // vertex groups - not supported
            
            else {
                Logger.w("unknown command: " + command);
                while (st.hasMoreTokens()) 
                    st.nextToken();
            }
            
            assert !st.hasMoreTokens();
        }
        
        if (mesh.startingPointsList.isEmpty())
            Logger.w("no starting points specified for "+filename);
        else
            Logger.i(mesh.startingPointsList.size()+" starting points in "+filename);
        
        return mesh;
    }

}
