/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.logic;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import netacka3d.data.Direction;
import netacka3d.data.Segment;
import netacka3d.misc.Logger;

/**
 * Kontroler.
 * 
 * Kotroluje grę, gra = ciąg meczy.
 * Jeden kontroler obsługuje tylko 1 mecz jednocześnie.
 * Tę klasę (w czasie gry) widzi UI.
 * Kontroler może mieć przypisanego Playera. Na razie powiedzmy, że zawsze ma.
 * 
 * Ogólnie: "W rzeczywistej grze całym przebiegiem sterują kontrolery. 
 * Kontroler ma oddzielny wątek i nadzoruje liczenie gry, operacje sieciowe,
 * itp.
 * Kontroler ma na własność odpowiedni obiekt ServerGame / ClientGame 
 * jako prywatne pole. Jest on udostępniany na zewnątrz w kontrolowany 
 * sposób podczas powiadomień (w GameListener)."
 */
public abstract class Controller {
    
    //protected Game game;
    
    /** id gracza obslugiwanego przez ten kontroler */
    protected int myPlayerId = 0;
    
    /** obiekty nasluchujace, wolamy je tylko z gameThread! */
    private List<GameListener> listeners;

    public Controller() {
        listeners = Collections.synchronizedList(new LinkedList<GameListener>());
    }
    
    /**
     * Zmiana kierunku przez gracza.
     * @param dir nowy kierunek wstegi gracza
     */
    public void setPlayerDirection(Direction dir) {}
    
    
    public void registerGameListener(GameListener gL) {
        //TODO moze tu warto sprawdzic, czy to poczatek gry, zeby sie nie
        // okazalo, ze dodajemy listenera w polowie rundy
        listeners.add(gL);
    }
    
    public void unregisterGameListeners() {
        listeners.clear();
    }    

    /* ---------------- funkcje do uzytku w podklasach --------------- */
    
    /** uwaga: myPlayerId musi byc ustawiony przy wolaniu tej funkcji */
    protected void notifyStart() {
        //assert(!listeners.isEmpty()); //?
        for (GameListener gL : listeners) 
            gL.onStart(myPlayerId);
    }
    
    protected void notifyStep(boolean stateChanged, Game game) {
        for (GameListener gL : listeners) 
            gL.onStep(stateChanged, game);
    }
    
    protected void notifySegment(int playerId, int t, Segment segment) {
        Logger.d("powiadamiam listenerów: nowy segment, pId="+playerId+", t="+t);
        assert(playerId != 0);
        assert(segment != null);
        //assert(game.players.getPlayer(playerId).active);
        for (GameListener gL : listeners) 
            gL.onSegment(playerId, t, segment);
    }
    
    protected void notifyStop(String msg) {
        Logger.i("zawołano notifyStop(" + msg + ")");
        //Logger.i("stacktrace:" + Arrays.toString(Thread.currentThread().getStackTrace()));
        for (GameListener gL : listeners) 
            gL.onStop(msg);
    }
        
}
