/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.misc;

import java.util.Formatter;
import java.util.Locale;

public abstract class Logger {
    
    private static LoggerBackend backend;
    private static Severity verbosityLevel = Severity.INFO;
    
    private static long startedTime = 0;
    
    static Object mutex = new Object();
    
    private static StringBuilder sB;
    
    public enum Severity {
        DEBUG(1), INFO(2), WARNING(3), ERROR(4), CRITICAL(5);
        public final int num;
        private Severity(int num) { this.num = num; }
        public char getSymbol() {
            return this.toString().charAt(0);
        }
    }
    
    // funkcje do użytku w kodzie
    public static void d(String msg) { pprint(msg, Severity.DEBUG); }
    public static void i(String msg) { pprint(msg, Severity.INFO); }
    public static void w(String msg) { pprint(msg, Severity.WARNING); }
    public static void e(String msg) { pprint(msg, Severity.ERROR); }
    public static void c(String msg) { pprint(msg, Severity.CRITICAL); }
    
    public static void logException(Exception e) {
        pprint("Caught exception:", Severity.ERROR);
        e.printStackTrace();
    }
    
    /** Pretty-print */
    private synchronized static void pprint(String msg, Severity severity) {
        
        if (severity.num < verbosityLevel.num)
            return;
        
        if (backend == null) 
            setBackend(new SimpleBackend());
        
        Thread currentThread = Thread.currentThread();
        
        long timeDiff = System.currentTimeMillis() - startedTime; 
        
        /* stacktrace: 
         * [0] - getStackTrace()
         * [1] - pprint
         * [2] - d, i, w, e lub c
         * [3] - miejsce w kodzie, które zawołało tą funkcję
         */
        StackTraceElement calledFrom = currentThread.getStackTrace()[3];
        
        // ten kod jest taki okropny, bo String.format jest strasznie wolne
        // na Androidzie
        sB.setLength(0);
        sB.append(timeDiff);
        sB.append(" [");
        sB.append(severity.getSymbol());
        sB.append("] ");
        sB.append(currentThread.getName());
        sB.append(" (");
        sB.append(calledFrom.getFileName());
        sB.append(':');
        sB.append(calledFrom.getLineNumber());
        sB.append(')');
        while (sB.length() < 50) 
            sB.append(' ');
        if (severity == Severity.DEBUG)
            sB.append("   ");
        sB.append(msg);
        sB.append("\n");
        
        msg = sB.toString();
        
        synchronized (mutex) {
            backend.log(msg, severity);    
        }
    }

    // w tej metodzie robimy też inicjację całego Loggera
    public synchronized static void setBackend(LoggerBackend b) {
        assert b != null;
        backend = b;
        sB = new StringBuilder();
        if (startedTime == 0)
            startedTime = System.currentTimeMillis();
    }
    
    public synchronized static void setVerbosity(Severity level) {
        verbosityLevel = level;
    }
    
    
    
    //-------------------------------------------------------------
    
    private static class SimpleBackend implements LoggerBackend {
        public void log(String msg, Severity severity) {
            if (severity.num >= Severity.WARNING.num) {
                System.err.print(msg);
                System.err.flush();
            }
            else {
                System.out.print(msg);
                System.out.flush();
            }
        }
    }
    
    //-------------------------------------------------------------
    
    public static class ExceptionHandler 
                        implements Thread.UncaughtExceptionHandler {
        public void uncaughtException(Thread t, Throwable e) {
            synchronized (mutex) {
                String divider = "------------------------------\n";
                backend.log(divider, Severity.CRITICAL);
                backend.log("Unhandled exception in thread " + t.getName() + "\n",
                            Severity.CRITICAL);
                e.printStackTrace();
                backend.log(divider, Severity.CRITICAL);
            }
            System.exit(1);
        }
    }
    
}
