/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.network;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.DatagramChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;

import netacka3d.data.MatchSettings;
import netacka3d.misc.Logger;

/**
 * Klasa zajmująca się wyszukiwaniem serwerów z grą w sieci lokalnej.
 * @author Leszek
 *
 */
public class ServerFinder {

    private ServerFinderListener listener;
    private DatagramChannel channelUDP;
    private FinderThread thread;
    private Selector selector;

    // czy wywołano stop 
    private volatile boolean closing = false;
    private volatile boolean retry = false;

    private final InetSocketAddress broadcastAddr;
    private final int BUF_SIZE = 6144;
    private final int TIMEOUT = 500;
    private ByteBuffer buf = ByteBuffer.allocate(BUF_SIZE);

    //    private final DatagramPacket broadcastPacket;
    //    private final byte[] sendBroadcastBuf = { UDPProtocol.LOOKING_FOR_SERVER, };


    /**
     * Konstruktor wyszukiwacza serwerów
     * @param listener listener
     */
    public ServerFinder(ServerFinderListener listener) 
    throws IOException {
        this.listener = listener;
        
        broadcastAddr = 
            new InetSocketAddress(InetAddress.getByName("255.255.255.255"),
                    UDPProtocol.DEFAULT_PORT);
        thread = new FinderThread();
        
        channelUDP = DatagramChannel.open();
        channelUDP.socket().bind(null);
        channelUDP.socket().setBroadcast(true);
        channelUDP.configureBlocking(false);
        selector = Selector.open();
        channelUDP.register(selector, SelectionKey.OP_READ);
        // broadcastPacket = new DatagramPacket(buf.array(), 1, broadcastAddr);
    }

    /**
     * Odpala nowy wątek. Wątek ten:
     * - Wysyła broadcastem komunikat GET_GAME_INFO
     * - Jak dostanie GAME_INFO, powiadamia listener.
     */
    public void start() {
        closing = false;
        retry = true;
        thread.start();
    }

    /**
     * Każe wątkowi dopytać się o serwery (tj. rozesłać broadcast) na nowo
     */
    public void refresh() {
        retry = true;
    }

    /**
     * Zabija wątek, kończy prace wątku wyszukującego i zamyka sockety.
     */
    public void stop() {
        closing = true;
    }

    /**
     * Rozsyła broadcast z zapytaniem o serwer
     * @throws IOException
     */
    private void sendBroadcast() throws IOException {
        buf.clear();
        buf.put(UDPProtocol.GET_GAME_INFO);
        buf.flip();
        channelUDP.send(buf, broadcastAddr);
    }

    /**
     * Oczekuje na odpowiedź na broadcast, po jej otrzymaniu nawiązuje
     * połączenie TCP, prosi o dane gry. Po ich otrzymaniu zamyka połączenie
     * i funkcja wraca.
     * @throws IOException
     */
    private void recResponse() throws IOException {
        buf.clear();
        InetSocketAddress addr = (InetSocketAddress) channelUDP.receive(buf);
        if (addr == null)
            return;
        buf.flip();
        if (buf.hasRemaining() && buf.get() == UDPProtocol.GAME_INFO) {
            int playersCount = buf.getInt();
            MatchSettings settings = 
                UDPProtocol.getJson(buf, MatchSettings.class);
            if (settings != null) {
                listener.onServer(addr.getHostName(), addr.getPort(), 
                        UDPProtocol.DEFAULT_PORT, 
                        settings, playersCount);
            }
        }
    }

    private void cleanup() {
        try {
            if (channelUDP != null)
                channelUDP.close();
        } catch (IOException e) {
            Logger.logException(e);
        }
    }


    private class FinderThread extends Thread {
        @Override
        public void run() {
            super.run();
            while (!closing) {
                try {
                    if (retry) {
                        retry = false;
                        sendBroadcast();
                    }
                    selector.select(TIMEOUT);
                    recResponse();
                } 
                catch (IOException e) {
                    Logger.logException(e);
                    cleanup();
                    return;
                }
            } // while
            cleanup();
        } // run
    }  // FinderThread


}
